;**************************************************************************
;* FileName	: RS485_SW.INC
;* Date 	: 14.12.2003
;* File version : v1.00
;*
;* Author	: MIKO, Imre
;* Company	:
;* Homepage	: http://majki.fw.hu
;* E-mail	: majki@fw.hu
;*
;*-------------------------------------------------------------------------
;*
;* Description	: functions for for rs485 communication
;*		  =====================================
;*		  RS485_SendByte
;*		  RS485_ReceiveByte
;*		  RS485_SendCS
;*
;*
;*-------------------------------------------------------------------------
;* Needed defines and includes in main-modul:
;* 
;* #define	RS485_DATADIR	PORTB,0		; out	(0=Rx, 1=Tx)
;* #define	RS485_TxD	PORTB,1		; out
;* #define	RS485_RxD	PORTB,2		; in
;*
;*-------------------------------------------------------------------------
;* Other possible defines in main-modul:
;*
;* #define RS485_OWN_DATADIR		; This module do not controll the datadir line
;*					; if this defined.
;*
;* #define DEBUG			; for some debug message
;* #define NO_RS485			; for fast simulation
;*
;**************************************************************************
;* Revision History:
;*
;* 1.00	  14/12/2003 	Initial Release
;*
;**************************************************************************

;***** VARIABLES **********************************************************

	ifdef INC_RS485
		cblock
		  RS485_BitNumber
		  RS485_SendBuffer
		  RS485_RecvBuffer
		  RS485_temp
		endc
	endif


;***** SEND SERIAL DATA ***************************************************
;* Send byte
;* Input : W				; byte
;* Output: -
;* Clock : ?
;**************************************************************************
	ifdef INC_RS485
RS485_SendByte	IT_DISABLE

		ifndef	RS485_OWN_DATADIR
		  bsf	RS485_DATADIR
		endif

		bcf	RS485_TxD		; start bit
		movwf	RS485_SendBuffer                          
		movlw	RS485_BITS		; x bit           
		movwf	RS485_BitNumber
		
RS485_sendloop	WAIT	RS485_BITTIME, 9

		bcf	STATUS,C		; 0 -> CARRY           
		rrf	RS485_SendBuffer,f
		skipc		
		bcf	RS485_TxD		; clear           
		skipnc
		bsf	RS485_TxD		; or set TxD

		decfsz	RS485_BitNumber,f
		goto	RS485_sendloop

		
		WAIT	RS485_BITTIME, 4

		bsf	RS485_TxD		; stop bit

		WAIT	(RS485_BITTIME * RS485_STOPBITS), 0

		ifndef	RS485_OWN_DATADIR
		  bcf	RS485_DATADIR
		endif

		IT_ENABLE
		return
	endif



;***** RECEIVE SERIAL DATA ************************************************
;* Receive byte
;* Input : -				; 
;* Output: received byte -> W		;
;* Clock : ?
;**************************************************************************
	ifdef	INC_RS485
RS485_ReceiveByte
		IT_DISABLE
		movlw	RS485_BITS		; x bit
		movwf	RS485_BitNumber

		clrf	RS485_RecvBuffer	; clear recieve buffer
RS485_waitrec	clrwdt
		btfsc	RS485_RxD
		goto	RS485_waitrec		; wait for start bit

		WAIT	(RS485_BITTIME + (RS485_BITTIME/3)), 11	; 1.333 bit

RS485_rloop	bcf	STATUS,C
		rrf	RS485_RecvBuffer,f
		btfsc	RS485_RxD
		bsf	RS485_RecvBuffer,7

		WAIT	RS485_BITTIME, 7

		decfsz	RS485_BitNumber,f	
		goto	RS485_rloop

		movf	RS485_RecvBuffer,w
		IT_ENABLE
		return
	endif



;***** RS485_SendCS *******************************************************
;* Send constant string
;* Input : -				; 
;* Output: received byte -> W		;
;* Clock : ?
;**************************************************************************
	ifdef	INC_RS485_SCS
RS485_SendCS	movwf	PCLATH

RS485_sendcs1	call	RS485_sendcs_getc
		testw
		skipnz
		return

		call	RS485_SendByte

		incf	RS485_temp,f
		skipnz
		incf	PCLATH,f
		goto	RS485_sendcs1

RS485_sendcs_getc
		movf	RS485_temp,w
		movwf	PCL		; computed goto!

	endif


;**************************************************************************



